/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPixelDescriptionActionHandler_h
#define __otbPixelDescriptionActionHandler_h

#include "otbImageWidgetActionHandler.h"

namespace otb
{
/** \class PixelDescriptionActionHandler
*   \brief Implements pixel reporting capabilities.
* 
*   \sa ImageWidgetController
*   \sa ImageWidgetActionHandler
*/

template <class TModel, class TView> 
class PixelDescriptionActionHandler
  : public ImageWidgetActionHandler
{
public:
  /** Standard class typedefs */
  typedef PixelDescriptionActionHandler       Self;
  typedef ImageWidgetActionHandler          Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;
  
  /** Method for creation through the object factory */
  itkNewMacro(Self);
  
  /** Runtime information */
  itkTypeMacro(PixelDescriptionActionHandler,ImageWidgetActionHandler);

  /** Model typedefs */
  typedef TModel                         ModelType;
  typedef typename ModelType::Pointer    ModelPointerType;
  typedef typename ModelType::IndexType  IndexType;

  /** View typedef */
  typedef TView                                     ViewType;
  typedef typename ViewType::Pointer                ViewPointerType;
  typedef typename ViewType::ImageWidgetPointerType WidgetPointerType;

   /** Handle widget event
   */
  virtual bool HandleWidgetEvent(std::string widgetId, int event)
  {
    if(m_Model.IsNotNull() &&  m_View.IsNotNull())
      {
      // Find the source widget
      WidgetPointerType sourceWidget;
      bool handle = false;
      if(widgetId == m_View->GetScrollWidget()->GetIdentifier() )
	{
	sourceWidget = m_View->GetScrollWidget();
	handle = true;
	}
      else if(widgetId == m_View->GetFullWidget()->GetIdentifier() )
	{
	sourceWidget = m_View->GetFullWidget();
	handle = true;
	}
      else if(widgetId == m_View->GetZoomWidget()->GetIdentifier() )
	{
	sourceWidget = m_View->GetZoomWidget();
	handle = true;
	}
      if(handle)
	{
	switch(event)
	  {
	  case FL_ENTER:
	  {
	  return true;
	  break;
	  }
	  case FL_LEAVE:
	  {
	  return true;
	  break;
	  }
	  case FL_MOVE:
	  {
	  // Get the hovered index
	  IndexType index;
	  index[0]=Fl::event_x();
	  index[1]=Fl::event_y();
	  // Convert to image index
	  index = sourceWidget->ScreenIndexToImageIndex(index);
	  // Communicate new index to model
	  m_Model->UpdatePixelDescription(index);
	  return true;
	  break;
	  }
	  default:
	  {
	  break;
	  }
	  }
	}
      }
    return false;
  }
  
  /** Set/Get the pointer to the model */
  itkSetObjectMacro(Model,ModelType);
  itkGetObjectMacro(Model,ModelType);

   /** Set/Get the pointer to the view */
  itkSetObjectMacro(View,ViewType);
  itkGetObjectMacro(View,ViewType);

protected:
  /** Constructor */
  PixelDescriptionActionHandler() : m_View(), m_Model()
  {}

  /** Destructor */
  virtual ~PixelDescriptionActionHandler(){}
  /** Printself method */
  void PrintSelf(std::ostream& os, itk::Indent indent) const
  {
    Superclass::PrintSelf(os,indent);
  }
 
private:
  PixelDescriptionActionHandler(const Self&);    // purposely not implemented
  void operator=(const Self&); // purposely not implemented

  // Pointer to the view
  ViewPointerType m_View;
  
  // Pointer to the model
  ModelPointerType m_Model;
  
}; // end class 
} // end namespace otb
#endif


